@@ -4,24 +4,24 @@
 
 // Copyright Aleksey Gurtovoy 2003-2004
 //
-// Distributed under the Boost Software License, Version 1.0. 
-// (See accompanying file LICENSE_1_0.txt or copy at 
+// Distributed under the Boost Software License, Version 1.0.
+// (See accompanying file LICENSE_1_0.txt or copy at
 // http://www.boost.org/LICENSE_1_0.txt)
 //
 // See http://www.boost.org/libs/mpl for documentation.
 
-// $Source: /cvsroot/boost/boost/boost/mpl/numeric_cast.hpp,v $
-// $Date: 2004/12/01 02:44:52 $
-// $Revision: 1.4.2.1 $
+// $Source$
+// $Date: 2005-06-26 19:18:33 +0200 (Sun, 26 Jun 2005) $
+// $Revision: 29796 $
 
 #include <boost/mpl/aux_/config/msvc.hpp>
 #include <boost/mpl/aux_/config/workaround.hpp>
 
-// agurt 21/sep/04: portability macro for the sake of MSVC 6.x-7.0; 
+// agurt 21/sep/04: portability macro for the sake of MSVC 6.x-7.0;
 // resolves conflicts with 'boost::numeric_cast' function template.
-// use it in your own code _only_ if you care about compatibility with 
+// use it in your own code _only_ if you care about compatibility with
 // these outdated compilers!
-#if BOOST_WORKAROUND(BOOST_MSVC, <= 1300)
+#if BOOST_WORKAROUND(BOOST_MSVC, <= 1300) || BOOST_WORKAROUND(__BORLANDC__, BOOST_TESTED_AT(0x570) )
 #   define BOOST_MPL_AUX_NUMERIC_CAST numeric_cast_
 #else
 #   define BOOST_MPL_AUX_NUMERIC_CAST numeric_cast
