@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License ver. 2+
-// $Id: GAfit.cpp 503 2009-06-09 15:47:31Z wojdyr $
+// $Id: GAfit.cpp 522 2009-07-27 12:25:56Z wojdyr $
 
 #include <stdlib.h>
 #include <time.h>
@@ -122,7 +122,7 @@ void GAfit::autoplot_in_autoiter()
 {
     const Individual& indiv = is_index(autoplot_indiv_nr, *pop)
                                     ? (*pop)[autoplot_indiv_nr] : best_indiv;
-    iteration_plot(indiv.g);
+    iteration_plot(indiv.g, true, indiv.raw_score);
 }
 
 void GAfit::mutation()
