@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License ver. 2+
-// $Id: NMfit.cpp 503 2009-06-09 15:47:31Z wojdyr $
+// $Id: NMfit.cpp 522 2009-07-27 12:25:56Z wojdyr $
 
 #include <stdlib.h>
 #include <time.h>
@@ -76,12 +76,12 @@ void NMfit::autoiter()
 {
     fp convergence = F->get_settings()->get_f("nm-convergence");
     wssr_before = compute_wssr(a_orig, dmdm_);
-    F->msg ("WSSR before starting simplex fit: " + S(wssr_before));
+    F->msg("WSSR before starting simplex fit: " + S(wssr_before));
     for (int iter = 0; !termination_criteria(iter, convergence); ++iter) {
-        iteration_plot(best->a);
         iter_nr++;
         change_simplex();
         find_best_worst();
+        iteration_plot(best->a, true, best->wssr);
     }
     post_fit (best->a, best->wssr);
 }
