@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License ver. 2+
-// $Id: datatrans2.h 503 2009-06-09 15:47:31Z wojdyr $
+// $Id: datatrans2.h 524 2009-08-02 23:48:19Z wojdyr $
 
 #ifndef FITYK__DATATRANS2__H__
 #define FITYK__DATATRANS2__H__
@@ -111,7 +111,7 @@ enum {
 
 struct push_double
 {
-    void operator()(const double& n) const;
+    void operator()(const double& d) const;
 };
 
 struct push_the_double: public push_double
@@ -126,8 +126,7 @@ struct push_the_double: public push_double
 #ifndef STANDALONE_DATATRANS
 struct push_var: public push_double
 {
-    void operator()(char const* a, char const* b) const
-     { push_double::operator()(AL->find_variable(string(a+1,b))->get_value()); }
+    void operator()(char const* a, char const* b) const;
 };
 
 struct push_func_param: public push_double
