@@ -1,17 +1,19 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License ver. 2+
-// $Id: fit.h 503 2009-06-09 15:47:31Z wojdyr $
+// $Id: fit.h 525 2009-08-03 10:51:18Z wojdyr $
 
 #ifndef FITYK__FIT__H__
 #define FITYK__FIT__H__
 #include <vector>
 #include <map>
 #include <string>
+#include <time.h>
 #include "common.h"
 
 
 class DataAndModel;
 class Ftk;
+class Variable;
 
 ///   interface of fitting method and implementation of common functions
 class Fit
@@ -44,10 +46,13 @@ public:
     // pretty-print matrix m x n stored in vec. `mname' is name/comment.
     std::string print_matrix (const std::vector<fp>& vec,
                                      int m, int n, const char *mname);
+    fp compute_r_squared(std::vector<fp> const &A,
+                         std::vector<DataAndModel*> const& dms);
 protected:
     Ftk *F;
     std::vector<DataAndModel*> dmdm_;
     int evaluations;
+    int max_evaluations_;
     int max_iterations; //it is set before calling autoiter()
     int iter_nr;
     fp wssr_before;
@@ -58,18 +63,19 @@ protected:
     virtual fp init() = 0; // called before autoiter()
     virtual void autoiter() = 0;
     bool common_termination_criteria(int iter);
-    fp compute_wssr(std::vector<fp> const &A,
-                    std::vector<DataAndModel*> const& dms, bool weigthed=true)
-        { ++evaluations; return do_compute_wssr(A, dms, weigthed); }
-    fp compute_r_squared(std::vector<fp> const &A,
-                         std::vector<DataAndModel*> const& dms);
     void compute_derivatives(std::vector<fp> const &A,
                              std::vector<DataAndModel*> const& dms,
                              std::vector<fp>& alpha, std::vector<fp>& beta);
+    fp compute_wssr(std::vector<fp> const &A,
+                    std::vector<DataAndModel*> const& dms, bool weigthed=true)
+        { ++evaluations; return do_compute_wssr(A, dms, weigthed); }
     bool post_fit (const std::vector<fp>& aa, fp chi2);
     fp draw_a_from_distribution (int nr, char distribution = 'u', fp mult = 1.);
-    void iteration_plot(std::vector<fp> const &A);
+    void iteration_plot(std::vector<fp> const &A, bool changed, fp wssr);
 private:
+    time_t last_refresh_time_;
+    time_t start_time_;
+
     void compute_derivatives_for(DataAndModel const *dm,
                                  std::vector<fp>& alpha, std::vector<fp>& beta);
     void update_parameters(std::vector<DataAndModel*> const& dms);
@@ -90,8 +96,9 @@ public:
     std::string param_history_info() const;
     std::vector<fp> const& get_item(int n) const { return param_history[n]; }
     int get_active_nr() const { return param_hist_ptr; }
-private:
+protected:
     Ftk *F;
+private:
     std::vector<std::vector<fp> > param_history; /// old parameter vectors
     int param_hist_ptr; /// points to the current/last parameter vector
 };
@@ -101,14 +108,18 @@ private:
 class FitMethodsContainer : public ParameterHistoryMgr
 {
 public:
-    FitMethodsContainer(Ftk *F);
+    FitMethodsContainer(Ftk *F_);
     ~FitMethodsContainer();
     Fit* get_method(int n) const
-                    { assert(n >= 0 && n<size(methods)); return methods[n]; }
-    std::vector<Fit*> const& get_methods() const { return methods; }
+                    { assert(n >= 0 && n<size(methods_)); return methods_[n]; }
+    std::vector<Fit*> const& get_methods() const { return methods_; }
+    fp get_symmetric_error(Variable const* var);
+    void outdated_error_cache() { dirty_error_cache_ = true; }
 
 private:
-    std::vector<Fit*> methods;
+    std::vector<Fit*> methods_;
+    std::vector<fp> errors_cache_;
+    bool dirty_error_cache_;
 
     FitMethodsContainer (FitMethodsContainer const&); //disable
 };
