@@ -1,13 +1,13 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
-// Licence: GNU General Public License version 2
-// $Id: guess.h 351 2007-10-24 19:22:53Z wojdyr $
+// Licence: GNU General Public License ver. 2+
+// $Id: guess.h 503 2009-06-09 15:47:31Z wojdyr $
 
 #ifndef FITYK__GUESS__H__
 #define FITYK__GUESS__H__
 
 enum FunctionKind { fk_peak, fk_linear /*, fk_step*/, fk_unknown };
 
-class DataWithSum;
+class DataAndModel;
 class Ftk;
 class Data;
 
@@ -15,7 +15,7 @@ class Data;
 class Guess
 {
 public:
-    Guess(Ftk const *F_, DataWithSum const* ds_);
+    Guess(Ftk const *F_, DataAndModel const* dm_);
     std::string get_guess_info(std::vector<std::string> const& range);
     void guess(std::string const& name, std::string const& function,
                std::vector<std::string> const& range,
@@ -26,7 +26,7 @@ protected:
     // only these peaks/functions are considered
     std::vector<int> real_peaks;
 
-    void estimate_peak_parameters (fp range_from, fp range_to, 
+    void estimate_peak_parameters (fp range_from, fp range_to,
                                    fp *center, fp *height, fp *area, fp *fwhm);
     void estimate_linear_parameters(fp range_from, fp range_to,
                                     fp *slope, fp *intercept, fp *avgy);
@@ -34,19 +34,19 @@ protected:
     void get_point_range(fp range_from, fp range_to, int &l_bor, int &r_bor);
 
     fp my_y (int n);
-    fp data_area (int from, int to); 
+    fp data_area (int from, int to);
     int max_data_y_pos (int from, int to);
     fp compute_data_fwhm (int from, int max_pos, int to, fp level);
-    void parse_range(std::vector<std::string> const& range, 
+    void parse_range(std::vector<std::string> const& range,
                      fp& range_from, fp& range_to);
     void remove_peak(std::string const& name);
 };
 
 
-bool is_function_guessable(std::string const& formula, 
+bool is_function_guessable(std::string const& formula,
                            bool check_defvalue=true);
 
-bool is_function_guessable(std::vector<std::string> const& vars, 
+bool is_function_guessable(std::vector<std::string> const& vars,
                            std::vector<std::string> const& defv,
                            FunctionKind* fk);
 
