@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License ver. 2+
-// $Id: cmn.h 503 2009-06-09 15:47:31Z wojdyr $
+// $Id: cmn.h 522 2009-07-27 12:25:56Z wojdyr $
 #ifndef FITYK_WX_CMN_H_
 #define FITYK_WX_CMN_H_
 
@@ -17,6 +17,8 @@ enum MouseModeEnum { mmd_zoom, mmd_bg, mmd_add, mmd_range, mmd_peak };
 // plot types, e.g. to tell where is mouse
 enum PlotTypeEnum { pte_main, pte_aux };
 
+// used in refresh_plots()
+enum WhichPlot { kMainPlot, kAllPlots };
 
 inline wxString pchar2wx(char const* pc)
 {
