@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License ver. 2+
-// $Id: dataedit.cpp 503 2009-06-09 15:47:31Z wojdyr $
+// $Id: dataedit.cpp 519 2009-07-02 20:55:30Z wojdyr $
 
 /// In this file:
 ///  Data Editor (DataEditorDlg) and helpers
@@ -64,7 +64,7 @@ public:
             case 1: return p.x;
             case 2: return p.y;
             case 3: return p.sigma;
-            default: assert(0);
+            default: assert(0); return 0.;
         }
     }
 
@@ -103,7 +103,7 @@ public:
             case 1: return wxT("x");
             case 2: return wxT("y");
             case 3: return wxT("sigma");
-            default: assert(0);
+            default: assert(0); return wxEmptyString;
         }
     }
 
