@@ -1,13 +1,13 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
-// Licence: GNU General Public License version 2
-// $Id: dialogs.h 403 2008-03-10 15:02:28Z wojdyr $
+// Licence: GNU General Public License ver. 2+
+// $Id: dialogs.h 503 2009-06-09 15:47:31Z wojdyr $
 
 #ifndef FITYK__WX_DLG__H__
 #define FITYK__WX_DLG__H__
 
 #include <vector>
 #include <wx/spinctrl.h>
-#include <wx/listctrl.h> 
+#include <wx/listctrl.h>
 
 #include "cmn.h"
 
@@ -19,8 +19,8 @@ public:
     void OnDownButton         (wxCommandEvent& event);
     void OnComputeWssrButton  (wxCommandEvent& event);
     void OnSelectedItem       (wxListEvent&    event);
-    void OnActivatedItem      (wxListEvent&    event); 
-    void OnViewSpinCtrlUpdate (wxSpinEvent&    event); 
+    void OnActivatedItem      (wxListEvent&    event);
+    void OnViewSpinCtrlUpdate (wxSpinEvent&    event);
     void OnClose (wxCommandEvent& ) { close_it(this); }
 protected:
     int view[4], view_max;
@@ -43,7 +43,7 @@ public:
     void OnSpinEvent(wxSpinEvent &) { update_unlimited(); }
     void OnChangeDsOrMethod(wxCommandEvent&) { update_allow_continue(); }
 private:
-    wxRadioBox* ds_rb;
+    wxRadioBox* data_rb;
     wxChoice* method_c;
     wxCheckBox* initialize_cb, *autoplot_cb;
     SpinCtrl *maxiter_sc, *maxeval_sc;
@@ -61,7 +61,7 @@ bool export_data_dlg(wxWindow *parent, bool load_exported=false);
 class DataExportDlg : public wxDialog
 {
 public:
-    DataExportDlg(wxWindow* parent, wxWindowID id, int ds_);
+    DataExportDlg(wxWindow* parent, wxWindowID id, int data_idx);
     void OnRadioChanged(wxCommandEvent&) { on_widget_change(); }
     void OnInactiveChanged(wxCommandEvent&) { on_widget_change(); }
     void OnTextChanged(wxCommandEvent&);
@@ -69,7 +69,7 @@ public:
     void on_widget_change();
     std::string get_columns() { return wx2s(text->GetValue()); }
 protected:
-    int ds;
+    int data_idx_;
     wxRadioBox *rb;
     wxCheckBox *inactive_cb;
     wxTextCtrl *text;
@@ -81,17 +81,6 @@ protected:
 };
 
 
-/// dialog Help->About
-class AboutDlg : public wxDialog
-{
-public:
-    AboutDlg(wxWindow* parent);
-    void OnTextURL(wxTextUrlEvent& event);
-private:
-    wxTextCtrl *txt;
-    DECLARE_EVENT_TABLE()
-};
-
 class DataListPlusText;
 
 class MergePointsDlg : public wxDialog
