@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License ver. 2+
-// $Id: dload.cpp 503 2009-06-09 15:47:31Z wojdyr $
+// $Id: dload.cpp 522 2009-07-27 12:25:56Z wojdyr $
 
 /// In this file:
 ///  Custom Data Load Dialog (DLoadDlg) and helpers
@@ -55,7 +55,7 @@ public:
     PreviewPlot(wxWindow* parent)
         : BufferedPanel(parent), block_nr(0), idx_x(1), idx_y(2),
           data_updated(false)
-        { backgroundCol = *wxBLACK; }
+        { set_bg_color(*wxBLACK); }
 
     void OnPaint(wxPaintEvent &event);
     void draw(wxDC &dc, bool);
