@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License ver. 2+
-// $Id: frame.cpp 503 2009-06-09 15:47:31Z wojdyr $
+// $Id: frame.cpp 535 2009-08-20 00:09:43Z wojdyr $
 
 #include <wx/wxprec.h>
 #ifdef __BORLANDC__
@@ -23,9 +23,6 @@
 #include <wx/msgout.h>
 #include <wx/metafile.h>
 #include <wx/dir.h>
-#ifdef __WXMSW__
-# include <wx/help.h>
-#endif
 #include <wx/stdpaths.h>
 
 #include <algorithm>
@@ -50,6 +47,7 @@
 #include "sdebug.h"
 #include "setdlg.h"
 #include "statbar.h"
+#include "fitinfo.h"
 #include "inputline.h"
 #include "app.h"
 #include "../common.h"
@@ -219,7 +217,6 @@ enum {
     ID_ft_v_pr                 ,
     ID_ft_b_strip              ,
     ID_ft_f_run                ,
-    ID_ft_f_cont               ,
     ID_ft_f_undo               ,
     ID_ft_s_aa                 ,
     ID_ft_sideb                ,
@@ -264,6 +261,7 @@ BEGIN_EVENT_TABLE(FFrame, wxFrame)
 
     EVT_UPDATE_UI_RANGE (ID_F_M, ID_F_M_END, FFrame::OnFMethodUpdate)
     EVT_MENU_RANGE (ID_F_M, ID_F_M_END, FFrame::OnFOneOfMethods)
+    EVT_UPDATE_UI (ID_F_RUN,    FFrame::OnMenuFitRunUpdate)
     EVT_MENU (ID_F_RUN,         FFrame::OnFRun)
     EVT_MENU (ID_F_INFO,        FFrame::OnFInfo)
     EVT_UPDATE_UI (ID_F_UNDO, FFrame::OnMenuFitUndoUpdate)
@@ -814,18 +812,13 @@ void FFrame::update_menu_previous_zooms()
 
 void FFrame::OnShowHelp(wxCommandEvent&)
 {
-#ifdef __WXMSW__
-    wxHelpController help;
-    wxString help_path = get_full_path_of_help_file(wxT("fitykhelp.chm"));
-    help.Initialize(help_path);
-    help.DisplayContents();
-#else
-    wxString help_path = get_full_path_of_help_file(wxT("fitykhelp.html"));
+    wxString help_file = wxString(wxT("html")) + wxFILE_SEP_PATH
+                         + wxT("fityk-manual.html");
+    wxString help_path = get_full_path_of_help_file(help_file);
     bool r = wxLaunchDefaultBrowser(wxT("file://") + help_path);
     if (!r)
         wxMessageBox(wxT("Can't open browser.\n Manual is here:\n") + help_path,
                      wxT("Manual"), wxOK|wxICON_INFORMATION);
-#endif
 }
 
 void FFrame::OnAbout(wxCommandEvent&)
@@ -837,10 +830,10 @@ void FFrame::OnAbout(wxCommandEvent&)
 
 void FFrame::OnContact(wxCommandEvent&)
 {
-    wxString url = wxT("http://www.unipress.waw.pl/fityk/contact.html");
+    wxString url = wxT("http://groups.google.com/group/fityk-users/topics");
     bool r = wxLaunchDefaultBrowser(url);
     if (!r)
-        wxMessageBox(wxT("Read instructions at:\n") + url,
+        wxMessageBox(wxT("Read instructions at the programs webpage.\n"),
                      wxT("feedback"), wxOK|wxICON_INFORMATION);
 }
 
@@ -1059,6 +1052,11 @@ void FFrame::OnFMethodUpdate (wxUpdateUIEvent& event)
     event.Check (ID_F_M + n == event.GetId());
 }
 
+void FFrame::OnMenuFitRunUpdate(wxUpdateUIEvent& event)
+{
+    event.Enable(!ftk->get_parameters().empty());
+}
+
 void FFrame::OnMenuFitUndoUpdate(wxUpdateUIEvent& event)
 {
     event.Enable(ftk->get_fit_container()->can_undo());
@@ -1094,7 +1092,10 @@ void FFrame::OnFRun (wxCommandEvent&)
 
 void FFrame::OnFInfo (wxCommandEvent&)
 {
-    ftk->exec("info fit" + get_in_datasets());
+    FitInfoDlg dlg(this, -1);
+    if (dlg.Initialize())
+        dlg.ShowModal();
+    //ftk->exec("info fit" + get_in_datasets());
 }
 
 void FFrame::OnFUndo (wxCommandEvent&)
@@ -1380,19 +1381,19 @@ void FFrame::OnUndoBg(wxCommandEvent&)
 void FFrame::OnClearBg(wxCommandEvent&)
 {
     plot_pane->get_bg_manager()->forget_background();
-    refresh_plots(false, true);
+    refresh_plots(false, kMainPlot);
 }
 
 void FFrame::OnConvexHullBg(wxCommandEvent&)
 {
     plot_pane->get_bg_manager()->set_as_convex_hull();
-    refresh_plots(false, true);
+    refresh_plots(false, kMainPlot);
 }
 
 void FFrame::OnSplineBg(wxCommandEvent& event)
 {
     plot_pane->get_bg_manager()->set_spline_bg(event.IsChecked());
-    refresh_plots(false, true);
+    refresh_plots(false, kMainPlot);
 }
 
 void FFrame::SwitchToolbar(bool show)
@@ -1672,7 +1673,7 @@ void FFrame::OnPrintToClipboard(wxCommandEvent&)
 {
 #if wxUSE_METAFILE
     wxMetafileDC dc;
-    if (dc.Ok()) {
+    if (dc.IsOk()) {
         do_print_plots(&dc, print_mgr);
         wxMetafile *mf = dc.Close();
         if (mf) {
@@ -1726,9 +1727,9 @@ void FFrame::output_text(OutputStyle style, const string& str)
     io_pane->output_win->append_text(style, s2wx(str));
 }
 
-void FFrame::refresh_plots(bool now, bool only_main)
+void FFrame::refresh_plots(bool now, WhichPlot which_plot)
 {
-    plot_pane->refresh_plots(now, only_main);
+    plot_pane->refresh_plots(now, which_plot);
 }
 
 void FFrame::draw_crosshair(int X, int Y)
@@ -1766,7 +1767,7 @@ void FFrame::update_toolbar()
     if (!toolbar)
         return;
     toolbar->ToggleTool(ID_ft_b_strip, plot_pane->get_bg_manager()->can_undo());
-    //toolbar->EnableTool(ID_ft_f_cont, ftk->get_fit()->is_initialized());
+    toolbar->EnableTool(ID_ft_f_run, !ftk->get_parameters().empty());
     toolbar->EnableTool(ID_ft_f_undo, ftk->get_fit_container()->can_undo());
     toolbar->EnableTool(ID_ft_v_pr, !plot_pane->get_zoom_hist().empty());
 }
@@ -1984,9 +1985,6 @@ FToolBar::FToolBar (wxFrame *parent, wxWindowID id)
     AddTool(ID_ft_f_run, wxT("Run"),
             wxBitmap(run_fit_xpm), wxNullBitmap, wxITEM_NORMAL,
             wxT("Start fitting"), wxT("Start fitting sum to data"));
-    //AddTool(ID_ft_f_cont, wxT("Continue"),
-    //        wxBitmap(cont_fit_xpm), wxNullBitmap, wxITEM_NORMAL,
-    //        wxT("Continue fitting"), wxT("Continue fitting sum to data"));
     AddTool(ID_ft_f_undo, wxT("Undo"),
              wxBitmap(undo_fit_xpm), wxNullBitmap, wxITEM_NORMAL,
              wxT("Undo fitting"), wxT("Previous set of parameters"));
@@ -2036,9 +2034,6 @@ void FToolBar::OnClickTool (wxCommandEvent& event)
         case ID_ft_f_run :
             ftk->exec("fit" + frame->get_in_datasets());
             break;
-        case ID_ft_f_cont:
-            ftk->exec("fit+");
-            break;
         case ID_ft_f_undo:
             ftk->exec("fit undo");
             break;
