@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License ver. 2+
-// $Id: frame.h 503 2009-06-09 15:47:31Z wojdyr $
+// $Id: frame.h 527 2009-08-03 11:46:45Z wojdyr $
 
 #ifndef FITYK__WX_GUI__H__
 #define FITYK__WX_GUI__H__
@@ -9,7 +9,7 @@
 #include <wx/spinctrl.h>
 
 #include <wx/filename.h>
-#include "cmn.h"  // Output_style_enum
+#include "cmn.h"  // enums
 #include "../common.h" //s2wx
 
 //struct z_names_type;
@@ -86,6 +86,7 @@ public:
     void OnSExport       (wxCommandEvent& event);
 
     void OnFMethodUpdate (wxUpdateUIEvent& event);
+    void OnMenuFitRunUpdate (wxUpdateUIEvent& event);
     void OnMenuFitUndoUpdate (wxUpdateUIEvent& event);
     void OnMenuFitRedoUpdate (wxUpdateUIEvent& event);
     void OnMenuFitHistoryUpdate (wxUpdateUIEvent& event);
@@ -172,7 +173,7 @@ public:
     void output_text(OutputStyle style, std::string const& str);
     void change_zoom(const std::string& s);
     void scroll_view_horizontally(fp step);
-    void refresh_plots(bool update=false, bool only_main=false);
+    void refresh_plots(bool now, WhichPlot which_plot);
     void draw_crosshair(int X, int Y);
     void focus_input(wxKeyEvent& event);
     void edit_in_input(std::string const& s);
