@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License ver. 2+
-// $Id: plot.cpp 510 2009-06-14 16:18:01Z wojdyr $
+// $Id: plot.cpp 522 2009-07-27 12:25:56Z wojdyr $
 
 /// In this file:
 ///  FPlot, the base class for MainPlot and AuxPlot
@@ -346,7 +346,7 @@ void FPlot::change_tics_font()
         wxFontData retData = dialog.GetFontData();
         ticsFont = retData.GetChosenFont();
         xAxisCol = retData.GetColour();
-        refresh(false);
+        refresh();
     }
 }
 
