@@ -1,6 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
 // Licence: GNU General Public License ver. 2+
-// $Id: statbar.h 511 2009-06-14 20:11:21Z wojdyr $
+// $Id: statbar.h 525 2009-08-03 10:51:18Z wojdyr $
 
 #ifndef FITYK_WX_STATBAR_H_
 #define FITYK_WX_STATBAR_H_
@@ -9,7 +9,7 @@
 #include <string>
 #include <wx/config.h>
 
-#include "cmn.h" //SpinCtrl, close_it
+#include "cmn.h" //SpinCtrl
 
 
 // used by MainPlot to set hints on FStatusBar
